/* vacScanRecord.c */
/****************************************************************
      Author :                     Greg Nawrocki
      Date:                        1-29-97

 This is the record support code for the vacScan record.
 It is designed to provide EPICS record support for a Spectra
 VacScan RGA.  The RGA in itself is a rather complex state
 machine controlled with this record.
****************************************************************/
 
/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/

#include <vxWorks.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <math.h>

#include "alarm.h"
#include "dbDefs.h"
#include "dbAccess.h"
#include "dbEvent.h"
#include "dbFldTypes.h"
#include "devSup.h"
#include "errMdef.h"
#include "recSup.h"
#include "special.h"
#include "dbScan.h"
#define GEN_SIZE_OFFSET
#include "vacScanRecord.h"
#undef  GEN_SIZE_OFFSET
#include "vacScanDefs.h"

int recVacScanDebug = 0;
/***** recVacScanDebug information *****/
/** recVacScanDebug >= 0 --- initialization information **/
/** recVacScanDebug >= 5 --- simulation mode **/
/** recVacScanDebug >= 10 -- alarm condition **/
/** recVacScanDebug >= 15 -- changed fields due to command output **/
/** recVacScanDebug >= 20 -- alarms that are set **/
/** recVacScanDebug >= 25 -- function calls **/

/* Create RSET - Record Support Entry Table*/
static long init_record(), process(), special();
static long get_graphic_double(), get_control_double(), get_alarm_double();
static long get_precision();
struct rset vacScanRSET = {
    RSETNUMBER,
    NULL,	/* report */
    NULL,	/* initialize */
    init_record,	/* init_record */
    process,	/* process */
    special,	/* special */
    NULL,	/* get_value */
    NULL,	/* cvt_dbaddr */
    NULL,	/* get_array_info */
    NULL,	/* put_array_info */
    NULL,	/* get_units */
    get_precision,	/* get_precision */
    NULL,	/* get_enum_str */
    NULL,	/* get_enum_strs */
    NULL,	/* put_enum_str */
    get_graphic_double,	/* get_graphic_double */
    get_control_double,	/* get_control_double */
    get_alarm_double /* get_alarm_double */
};

struct vacScandset {	/* vacScan dset */
    long number;
    DEVSUPFUN dev_report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;	/* (-1,0)=>(failure,success) */
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN proc_xact;	/* Always return 2, (don't convert) */
};
static void alarm(vacScanRecord *vsCtl);
static void monitor(vacScanRecord *vsCtl);

static long init_record(struct vacScanRecord *vsCtl, int pass)
{
    struct vacScandset *pdset;
    long status;

    if (recVacScanDebug)
	printf("recVacScan.c: Specra Vac Scan record initialization\n");
    if (pass == 0) return (0);
    vsCtl->flgs = 0;
    /********** Initialization of Record **********/
    if (recVacScanDebug) printf("recVacScan.c: Look for device support\n");
    /*** look for device support ***/
    if (!(pdset = (struct vacScandset *) (vsCtl->dset))) {
	recGblRecordError(S_dev_noDSET, (struct dbCommon *) vsCtl,
	    "vacScan: init_record");
	if (recVacScanDebug)
	    printf("recVacScan.c: Missing device support entry table\n");
	return (S_dev_noDSET);
    } else {
	if (recVacScanDebug)
	    printf("recVacScan.c: Device support entry table found\n");
    }
    /*** must have proc_xact function defined ***/
    if ((pdset->number < 5) || (pdset->proc_xact == NULL)) {
	recGblRecordError(S_dev_missingSup, (struct dbCommon *) vsCtl,
	    "vacScan: init_record");
	if (recVacScanDebug) printf("recVacScan.c: Missing device support\n");
	return (S_dev_missingSup);
    } else {
	if (recVacScanDebug) printf("recVacScan.c: Device support found\n");
    }
    /*** if everything is OK init the record ***/
    if (pdset->init_record) {
	if ((status = (*pdset->init_record) (vsCtl))) {
	    if (recVacScanDebug)
		printf("recVacScan.c: Failure in calling record initialization\n");
	    return (status);
	} else {
	    if (recVacScanDebug)
		printf("recVacScan.c: Record initialization complete\n");
	}
    }
    return (0);
}

static long process(struct vacScanRecord *vsCtl)
{
    struct vacScandset *pdset = (struct vacScandset *) (vsCtl->dset);
    long status;
    char pact = vsCtl->pact;
    if (recVacScanDebug >= 25)
	printf("recVacScan.c: Process(%s)\n", vsCtl->name);
    if ((pdset == NULL) || (pdset->proc_xact == NULL)) {
	vsCtl->pact = TRUE;
	recGblRecordError(S_dev_missingSup, (struct dbCommon *) vsCtl,
	    "read_vacScan");
	return (S_dev_missingSup);
    }
    if (!pact) {
	vsCtl->ival = vsCtl->val;
	vsCtl->imod = vsCtl->rmod;
	vsCtl->ibcf = vsCtl->rbcf;
	vsCtl->ibcs = vsCtl->rbcs;
	vsCtl->iafm = vsCtl->rafm;
	vsCtl->itri = vsCtl->rtri;
	vsCtl->iprg = vsCtl->rprg;
	vsCtl->idet = vsCtl->rdet;
	vsCtl->iagn = vsCtl->ragn;
	vsCtl->itpr = vsCtl->rtpr;
	vsCtl->ignr = vsCtl->rgnr;
	vsCtl->issp = vsCtl->rssp;
	vsCtl->ifs1 = vsCtl->rfs1;
	vsCtl->ifs2 = vsCtl->rfs2;
	vsCtl->ifsl = vsCtl->rfsl;
	vsCtl->ifon = vsCtl->rfon;
	vsCtl->idrs = vsCtl->rdrs;
	vsCtl->idup = vsCtl->rdup;
	vsCtl->idbc = vsCtl->rdbc;
	vsCtl->idlc = vsCtl->rdlc;
	vsCtl->idan = vsCtl->rdan;
	vsCtl->idvs = vsCtl->rdvs;
	vsCtl->idut = vsCtl->rdut;
	vsCtl->iddt = vsCtl->rddt;
	vsCtl->idal = vsCtl->rdal;
	vsCtl->iddg = vsCtl->rddg;
	vsCtl->idmc = vsCtl->rdmc;
	vsCtl->iddi = vsCtl->rddi;
	vsCtl->idtr = vsCtl->rdtr;
	vsCtl->ielm = vsCtl->nelm;
	vsCtl->ifsc = vsCtl->wfsc;
	vsCtl->i1pm = vsCtl->r1pm;
	vsCtl->i1pv = vsCtl->r1pv;
	vsCtl->i1pd = vsCtl->r1pd;
	vsCtl->i1pa = vsCtl->r1pa;
	vsCtl->i1pg = vsCtl->r1pg;
	vsCtl->i1pi = vsCtl->r1pi;
	vsCtl->i1ph = vsCtl->r1ph;
	vsCtl->i1pl = vsCtl->r1pl;
	vsCtl->i1pc = vsCtl->r1pc;
	vsCtl->i1tm = vsCtl->r1tm;
	vsCtl->i1tv = vsCtl->r1tv;
	vsCtl->i1td = vsCtl->r1td;
	vsCtl->i1ta = vsCtl->r1ta;
	vsCtl->i1tg = vsCtl->r1tg;
	vsCtl->i1ti = vsCtl->r1ti;
	vsCtl->i1th = vsCtl->r1th;
	vsCtl->i1tl = vsCtl->r1tl;
	vsCtl->i1tc = vsCtl->r1tc;
	vsCtl->i2pm = vsCtl->r2pm;
	vsCtl->i2pv = vsCtl->r2pv;
	vsCtl->i2pd = vsCtl->r2pd;
	vsCtl->i2pa = vsCtl->r2pa;
	vsCtl->i2pg = vsCtl->r2pg;
	vsCtl->i2pi = vsCtl->r2pi;
	vsCtl->i2ph = vsCtl->r2ph;
	vsCtl->i2pl = vsCtl->r2pl;
	vsCtl->i2pc = vsCtl->r2pc;
	vsCtl->i2tm = vsCtl->r2tm;
	vsCtl->i2tv = vsCtl->r2tv;
	vsCtl->i2td = vsCtl->r2td;
	vsCtl->i2ta = vsCtl->r2ta;
	vsCtl->i2tg = vsCtl->r2tg;
	vsCtl->i2ti = vsCtl->r2ti;
	vsCtl->i2th = vsCtl->r2th;
	vsCtl->i2tl = vsCtl->r2tl;
	vsCtl->i2tc = vsCtl->r2tc;
	vsCtl->i3pm = vsCtl->r3pm;
	vsCtl->i3pv = vsCtl->r3pv;
	vsCtl->i3pd = vsCtl->r3pd;
	vsCtl->i3pa = vsCtl->r3pa;
	vsCtl->i3pg = vsCtl->r3pg;
	vsCtl->i3pi = vsCtl->r3pi;
	vsCtl->i3ph = vsCtl->r3ph;
	vsCtl->i3pl = vsCtl->r3pl;
	vsCtl->i3pc = vsCtl->r3pc;
	vsCtl->i3tm = vsCtl->r3tm;
	vsCtl->i3tv = vsCtl->r3tv;
	vsCtl->i3td = vsCtl->r3td;
	vsCtl->i3ta = vsCtl->r3ta;
	vsCtl->i3tg = vsCtl->r3tg;
	vsCtl->i3ti = vsCtl->r3ti;
	vsCtl->i3th = vsCtl->r3th;
	vsCtl->i3tl = vsCtl->r3tl;
	vsCtl->i3tc = vsCtl->r3tc;
	vsCtl->i4pm = vsCtl->r4pm;
	vsCtl->i4pv = vsCtl->r4pv;
	vsCtl->i4pd = vsCtl->r4pd;
	vsCtl->i4pa = vsCtl->r4pa;
	vsCtl->i4pg = vsCtl->r4pg;
	vsCtl->i4pi = vsCtl->r4pi;
	vsCtl->i4ph = vsCtl->r4ph;
	vsCtl->i4pl = vsCtl->r4pl;
	vsCtl->i4pc = vsCtl->r4pc;
	vsCtl->i4tm = vsCtl->r4tm;
	vsCtl->i4tv = vsCtl->r4tv;
	vsCtl->i4td = vsCtl->r4td;
	vsCtl->i4ta = vsCtl->r4ta;
	vsCtl->i4tg = vsCtl->r4tg;
	vsCtl->i4ti = vsCtl->r4ti;
	vsCtl->i4th = vsCtl->r4th;
	vsCtl->i4tl = vsCtl->r4tl;
	vsCtl->i4tc = vsCtl->r4tc;
	vsCtl->i5pm = vsCtl->r5pm;
	vsCtl->i5pv = vsCtl->r5pv;
	vsCtl->i5pd = vsCtl->r5pd;
	vsCtl->i5pa = vsCtl->r5pa;
	vsCtl->i5pg = vsCtl->r5pg;
	vsCtl->i5pi = vsCtl->r5pi;
	vsCtl->i5ph = vsCtl->r5ph;
	vsCtl->i5pl = vsCtl->r5pl;
	vsCtl->i5pc = vsCtl->r5pc;
	vsCtl->i5tm = vsCtl->r5tm;
	vsCtl->i5tv = vsCtl->r5tv;
	vsCtl->i5td = vsCtl->r5td;
	vsCtl->i5ta = vsCtl->r5ta;
	vsCtl->i5tg = vsCtl->r5tg;
	vsCtl->i5ti = vsCtl->r5ti;
	vsCtl->i5th = vsCtl->r5th;
	vsCtl->i5tl = vsCtl->r5tl;
	vsCtl->i5tc = vsCtl->r5tc;
	vsCtl->i6pm = vsCtl->r6pm;
	vsCtl->i6pv = vsCtl->r6pv;
	vsCtl->i6pd = vsCtl->r6pd;
	vsCtl->i6pa = vsCtl->r6pa;
	vsCtl->i6pg = vsCtl->r6pg;
	vsCtl->i6pi = vsCtl->r6pi;
	vsCtl->i6ph = vsCtl->r6ph;
	vsCtl->i6pl = vsCtl->r6pl;
	vsCtl->i6pc = vsCtl->r6pc;
	vsCtl->i6tm = vsCtl->r6tm;
	vsCtl->i6tv = vsCtl->r6tv;
	vsCtl->i6td = vsCtl->r6td;
	vsCtl->i6ta = vsCtl->r6ta;
	vsCtl->i6tg = vsCtl->r6tg;
	vsCtl->i6ti = vsCtl->r6ti;
	vsCtl->i6th = vsCtl->r6th;
	vsCtl->i6tl = vsCtl->r6tl;
	vsCtl->i6tc = vsCtl->r6tc;
	vsCtl->ic0v = vsCtl->rc0v;
	vsCtl->ic1v = vsCtl->rc1v;
	vsCtl->ic2v = vsCtl->rc2v;
	vsCtl->ic3v = vsCtl->rc3v;
	vsCtl->ic4v = vsCtl->rc4v;
	vsCtl->ic5v = vsCtl->rc5v;
	vsCtl->ic6v = vsCtl->rc6v;
    }
    /*** call device support for processing ***/
    status = (*pdset->proc_xact) (vsCtl);
    /*** Device support is in asynchronous mode, let it finish ***/
    if (!pact && vsCtl->pact) return (0);
    vsCtl->pact = TRUE;
    recGblGetTimeStamp(vsCtl);
    /*** check for alarms ***/
    alarm(vsCtl);
    /*** check event list ***/
    monitor(vsCtl);
    /*** process the forward scan link record ***/
    recGblFwdLink(vsCtl);
    vsCtl->pact = FALSE;
    return (status);
}

static long get_precision(DBADDR *paddr, long *precision)
{
    struct vacScanRecord *pvs = (struct vacScanRecord *) paddr->precord;
    if (paddr->pfield == (void *) &pvs->r1pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r1tv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r2pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r2tv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r3pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r3tv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r4pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r4tv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r5pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r5tv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r6pv) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->r6tv) {
	*precision = 1;
	return (0);
    }
    if(paddr->pfield == (void *)&pvs->rc0v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc1v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc2v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc3v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc4v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc5v) {
	*precision = 1;
	return (0);
    }
    if (paddr->pfield == (void *) &pvs->rc6v) {
	*precision = 1;
	return (0);
    }
    *precision = 0;
    recGblGetPrec(paddr, precision);
    return (0);
}

static long special(DBADDR *pdbAddr, int after)
{
    void *p;
    struct vacScanRecord *vsCtl = (struct vacScanRecord *) (pdbAddr->precord);
    if (!after) return (0);
    scanOnce((void *)vsCtl);
    if (recVacScanDebug >= 25) printf("recVacScan.c: special()\n");
    /*** Make sure we have the proper special flag type spec'd ***/
    if (pdbAddr->special != SPC_MOD) {
	recGblDbaddrError(S_db_badChoice, pdbAddr, "vs: special");
	return (S_db_badChoice);
    }
    p = (void *) (pdbAddr->pfield);
    /*** Figure out which field has been changed ***/
    if (p == &(vsCtl->cmod)) {
	vsCtl->flgs |= VS_FLGS_CMOD;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CMOD changed to %hd\n",
	    vsCtl->name, vsCtl->cmod);
    }
    if (p == &(vsCtl->cloc)) {
	vsCtl->flgs |= VS_FLGS_CLOC;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CLOC changed to %hd\n",
	    vsCtl->name, vsCtl->cloc);
    }
    if (p == &(vsCtl->cxf1)) {
	vsCtl->flgs |= VS_FLGS_CXF1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CXF1 changed to %hd\n",
	    vsCtl->name, vsCtl->cxf1);
    }
    if (p == &(vsCtl->cxf2)) {
	vsCtl->flgs |= VS_FLGS_CXF2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CXF2 changed to %hd\n",
	    vsCtl->name, vsCtl->cxf2);
    }
    if (p == &(vsCtl->crst)) {
	vsCtl->flgs |= VS_FLGS_CRST;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CRST changed to %hd\n",
	    vsCtl->name, vsCtl->crst);
    }
    if (p == &(vsCtl->cylm)) {
	vsCtl->ymfg |= YMFG_LEAK;
	vsCtl->yfg |= YFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYLM changed to %hd\n",
	    vsCtl->name, vsCtl->cylm);
    }
    if (p == &(vsCtl->cyls)) {
	vsCtl->ymfg |= YMFG_LEAK;
	vsCtl->yfg |= YFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYLS changed to %hd\n",
	    vsCtl->name, vsCtl->cyls);
    }
    if (p == &(vsCtl->cylg)) {
	vsCtl->ymfg |= YMFG_LEAK;
	vsCtl->yfg |= YFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYLG changed to %f\n",
	    vsCtl->name, vsCtl->cylg);
    }
    if (p == &(vsCtl->cyli)) {
	vsCtl->ymfg |= YMFG_LEAK;
	vsCtl->yfg |= YFG_9;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYLI changed to %f\n",
	    vsCtl->name, vsCtl->cyli);
    }
    if (p == &(vsCtl->cypa)) {
	vsCtl->ymfg |= YMFG_PEAK;
	vsCtl->yfg |= YFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYPA changed to %hd\n",
	    vsCtl->name, vsCtl->cypa);
    }
    if (p == &(vsCtl->cyps)) {
	vsCtl->ymfg |= YMFG_PEAK;
	vsCtl->yfg |= YFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYPS changed to %hd\n",
	    vsCtl->name, vsCtl->cyps);
    }
    if (p == &(vsCtl->cybm)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBM changed to %hd\n",
	    vsCtl->name, vsCtl->cybm);
    }
    if (p == &(vsCtl->cyba)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBA changed to %hd\n",
	    vsCtl->name, vsCtl->cyba);
    }
    if (p == &(vsCtl->cybs)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBS changed to %hd\n",
	    vsCtl->name, vsCtl->cybs);
    }
    if (p == &(vsCtl->cybp)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBP changed to %f\n",
	    vsCtl->name, vsCtl->cybp);
    }
    if (p == &(vsCtl->cybt)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBT changed to %f\n",
	    vsCtl->name, vsCtl->cybt);
    }
    if (p == &(vsCtl->cybg)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBG changed to %f\n",
	    vsCtl->name, vsCtl->cybg);
    }
    if (p == &(vsCtl->cybh)) {
	vsCtl->ymfg |= YMFG_BAR;
	vsCtl->yfg |= YFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYBH changed to %hd\n",
	    vsCtl->name, vsCtl->cybh);
    }
    if (p == &(vsCtl->cyam)) {
	vsCtl->ymfg |= YMFG_ANALOG;
	vsCtl->yfg |= YFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYAM changed to %hd\n",
	    vsCtl->name, vsCtl->cyam);
    }
    if (p == &(vsCtl->cyaa)) {
	vsCtl->ymfg |= YMFG_ANALOG;
	vsCtl->yfg |= YFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYAA changed to %hd\n",
	    vsCtl->name, vsCtl->cyaa);
    }
    if (p == &(vsCtl->cyas)) {
	vsCtl->ymfg |= YMFG_ANALOG;
	vsCtl->yfg |= YFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYAS changed to %hd\n",
	    vsCtl->name, vsCtl->cyas);
    }
    if (p == &(vsCtl->cyap)) {
	vsCtl->ymfg |= YMFG_ANALOG;
	vsCtl->yfg |= YFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYAP changed to %f\n",
	    vsCtl->name, vsCtl->cyap);
    }
    if (p == &(vsCtl->cyag)) {
	vsCtl->ymfg |= YMFG_ANALOG;
	vsCtl->yfg |= YFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYAG changed to %f\n",
	    vsCtl->name, vsCtl->cyag);
    }
    if (p == &(vsCtl->cyta)) {
	vsCtl->ymfg |= YMFG_TREND;
	vsCtl->yfg |= YFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYTA changed to %hd\n",
	    vsCtl->name, vsCtl->cyta);
    }
    if (p == &(vsCtl->cyts)) {
	vsCtl->ymfg |= YMFG_TREND;
	vsCtl->yfg |= YFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYTS changed to %hd\n",
	    vsCtl->name, vsCtl->cyts);
    }
    if (p == &(vsCtl->cytn)) {
	vsCtl->ymfg |= YMFG_TREND;
	vsCtl->yfg |= YFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< CYTN changed to %f\n",
	    vsCtl->name, vsCtl->cytn);
    }
    if (p == &(vsCtl->c1pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PM changed to %hd\n",
	    vsCtl->name, vsCtl->c1pm);
    }
    if (p == &(vsCtl->c1pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PC changed to %hd\n",
	    vsCtl->name, vsCtl->c1pc);
    }
    if (p == &(vsCtl->c1ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PS changed to %f\n",
	    vsCtl->name, vsCtl->c1ps);
    }
    if (p == &(vsCtl->c1pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PG changed to %f\n",
	    vsCtl->name, vsCtl->c1pg);
    }
    if (p == &(vsCtl->c1pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PA changed to %hd\n",
	    vsCtl->name, vsCtl->c1pa);
    }
    if (p == &(vsCtl->c1pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PL changed to %f\n",
	    vsCtl->name, vsCtl->c1pl);
    }
    if (p == &(vsCtl->c1pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PN changed to %hd\n",
	    vsCtl->name, vsCtl->c1pn);
    }
    if (p == &(vsCtl->c1pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1PK changed to %hd\n",
	    vsCtl->name, vsCtl->c1pk);
    }
    if (p == &(vsCtl->c1tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1TM changed to %hd\n",
	    vsCtl->name, vsCtl->c1tm);
    }
    if (p == &(vsCtl->c1tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1TC changed to %hd\n",
	    vsCtl->name, vsCtl->c1tc);
    }
    if (p == &(vsCtl->c1ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1TS changed to %f\n",
	    vsCtl->name, vsCtl->c1ts);
    }
    if (p == &(vsCtl->c1tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1TG changed to %f\n",
	    vsCtl->name, vsCtl->c1tg);
    }
    if (p == &(vsCtl->c1ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_1;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C1TA changed to %hd\n",
	    vsCtl->name, vsCtl->c1ta);
    }
    if (p == &(vsCtl->c2pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PM changed to %hd\n",
	    vsCtl->name, vsCtl->c2pm);
    }
    if (p == &(vsCtl->c2pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PC changed to %hd\n",
	    vsCtl->name, vsCtl->c2pc);
    }
    if (p == &(vsCtl->c2ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PS changed to %f\n",
	    vsCtl->name, vsCtl->c2ps);
    }
    if (p == &(vsCtl->c2pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PG changed to %f\n",
	    vsCtl->name, vsCtl->c2pg);
    }
    if (p == &(vsCtl->c2pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PA changed to %hd\n",
	    vsCtl->name, vsCtl->c2pa);
    }
    if (p == &(vsCtl->c2pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PL changed to %f\n",
	    vsCtl->name, vsCtl->c2pl);
    }
    if (p == &(vsCtl->c2pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PN changed to %hd\n",
	    vsCtl->name, vsCtl->c2pn);
    }
    if (p == &(vsCtl->c2pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2PK changed to %hd\n",
	    vsCtl->name, vsCtl->c2pk);
    }
    if (p == &(vsCtl->c2tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2TM changed to %hd\n",
	    vsCtl->name, vsCtl->c2tm);
    }
    if (p == &(vsCtl->c2tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2TC changed to %hd\n",
	    vsCtl->name, vsCtl->c2tc);
    }
    if (p == &(vsCtl->c2ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2TS changed to %f\n",
	    vsCtl->name, vsCtl->c2ts);
    }
    if (p == &(vsCtl->c2tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2TG changed to %f\n",
	    vsCtl->name, vsCtl->c2tg);
    }
    if (p == &(vsCtl->c2ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_2;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C2TA changed to %hd\n",
	    vsCtl->name, vsCtl->c2ta);
    }
    if (p == &(vsCtl->c3pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PM changed to %hd\n",
	    vsCtl->name, vsCtl->c3pm);
    }
    if (p == &(vsCtl->c3pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PC changed to %hd\n",
	    vsCtl->name, vsCtl->c3pc);
    }
    if (p == &(vsCtl->c3ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PS changed to %f\n",
	    vsCtl->name, vsCtl->c3ps);
    }
    if (p == &(vsCtl->c3pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PG changed to %f\n",
	    vsCtl->name, vsCtl->c3pg);
    }
    if (p == &(vsCtl->c3pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PA changed to %hd\n",
	    vsCtl->name, vsCtl->c3pa);
    }
    if (p == &(vsCtl->c3pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PL changed to %f\n",
	    vsCtl->name, vsCtl->c3pl);
    }
    if (p == &(vsCtl->c3pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PN changed to %hd\n",
	    vsCtl->name, vsCtl->c3pn);
    }
    if (p == &(vsCtl->c3pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3PK changed to %hd\n",
	    vsCtl->name, vsCtl->c3pk);
    }
    if (p == &(vsCtl->c3tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3TM changed to %hd\n",
	    vsCtl->name, vsCtl->c3tm);
    }
    if (p == &(vsCtl->c3tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3TC changed to %hd\n",
	    vsCtl->name, vsCtl->c3tc);
    }
    if (p == &(vsCtl->c3ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3TS changed to %f\n",
	    vsCtl->name, vsCtl->c3ts);
    }
    if (p == &(vsCtl->c3tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3TG changed to %f\n",
	    vsCtl->name, vsCtl->c3tg);
    }
    if (p == &(vsCtl->c3ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_3;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C3TA changed to %hd\n",
	    vsCtl->name, vsCtl->c3ta);
    }
    if (p == &(vsCtl->c4pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PM changed to %hd\n",
	    vsCtl->name, vsCtl->c4pm);
    }
    if (p == &(vsCtl->c4pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PC changed to %hd\n",
	    vsCtl->name, vsCtl->c4pc);
    }
    if (p == &(vsCtl->c4ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PS changed to %f\n",
	    vsCtl->name, vsCtl->c4ps);
    }
    if (p == &(vsCtl->c4pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PG changed to %f\n",
	    vsCtl->name, vsCtl->c4pg);
    }
    if (p == &(vsCtl->c4pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PA changed to %hd\n",
	    vsCtl->name, vsCtl->c4pa);
    }
    if (p == &(vsCtl->c4pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PL changed to %f\n",
	    vsCtl->name, vsCtl->c4pl);
    }
    if (p == &(vsCtl->c4pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PN changed to %hd\n",
	    vsCtl->name, vsCtl->c4pn);
    }
    if (p == &(vsCtl->c4pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4PK changed to %hd\n",
	    vsCtl->name, vsCtl->c4pk);
    }
    if (p == &(vsCtl->c4tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4TM changed to %hd\n",
	    vsCtl->name, vsCtl->c4tm);
    }
    if (p == &(vsCtl->c4tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4TC changed to %hd\n",
	    vsCtl->name, vsCtl->c4tc);
    }
    if (p == &(vsCtl->c4ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4TS changed to %f\n",
	    vsCtl->name, vsCtl->c4ts);
    }
    if (p == &(vsCtl->c4tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4TG changed to %f\n",
	    vsCtl->name, vsCtl->c4tg);
    }
    if (p == &(vsCtl->c4ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_4;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C4TA changed to %hd\n",
	    vsCtl->name, vsCtl->c4ta);
    }
    if (p == &(vsCtl->c5pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PM changed to %hd\n",
	    vsCtl->name, vsCtl->c5pm);
    }
    if (p == &(vsCtl->c5pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PC changed to %hd\n",
	    vsCtl->name, vsCtl->c5pc);
    }
    if (p == &(vsCtl->c5ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PS changed to %f\n",
	    vsCtl->name, vsCtl->c5ps);
    }
    if (p == &(vsCtl->c5pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PG changed to %f\n",
	    vsCtl->name, vsCtl->c5pg);
    }
    if (p == &(vsCtl->c5pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PA changed to %hd\n",
	    vsCtl->name, vsCtl->c5pa);
    }
    if (p == &(vsCtl->c5pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PL changed to %f\n",
	    vsCtl->name, vsCtl->c5pl);
    }
    if (p == &(vsCtl->c5pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PN changed to %hd\n",
	    vsCtl->name, vsCtl->c5pn);
    }
    if (p == &(vsCtl->c5pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5PK changed to %hd\n",
	    vsCtl->name, vsCtl->c5pk);
    }
    if (p == &(vsCtl->c5tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5TM changed to %hd\n",
	    vsCtl->name, vsCtl->c5tm);
    }
    if (p == &(vsCtl->c5tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5TC changed to %hd\n",
	    vsCtl->name, vsCtl->c5tc);
    }
    if (p == &(vsCtl->c5ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5TS changed to %f\n",
	    vsCtl->name, vsCtl->c5ts);
    }
    if (p == &(vsCtl->c5tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5TG changed to %f\n",
	    vsCtl->name, vsCtl->c5tg);
    }
    if (p == &(vsCtl->c5ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_5;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C5TA changed to %hd\n",
	    vsCtl->name, vsCtl->c5ta);
    }
    if (p == &(vsCtl->c6pm)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PM changed to %hd\n",
	    vsCtl->name, vsCtl->c6pm);
    }
    if (p == &(vsCtl->c6pc)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PC changed to %hd\n",
	    vsCtl->name, vsCtl->c6pc);
    }
    if (p == &(vsCtl->c6ps)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PS changed to %f\n",
	    vsCtl->name, vsCtl->c6ps);
    }
    if (p == &(vsCtl->c6pg)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PG changed to %f\n",
	    vsCtl->name, vsCtl->c6pg);
    }
    if (p == &(vsCtl->c6pa)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PA changed to %hd\n",
	    vsCtl->name, vsCtl->c6pa);
    }
    if (p == &(vsCtl->c6pl)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_6;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PL changed to %f\n",
	    vsCtl->name, vsCtl->c6pl);
    }
    if (p == &(vsCtl->c6pn)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_7;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PN changed to %hd\n",
	    vsCtl->name, vsCtl->c6pn);
    }
    if (p == &(vsCtl->c6pk)) {
	vsCtl->zmfg |= ZMFG_PEAK;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_8;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6PK changed to %hd\n",
	    vsCtl->name, vsCtl->c6pk);
    }
    if (p == &(vsCtl->c6tm)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_1;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6TM changed to %hd\n",
	    vsCtl->name, vsCtl->c6tm);
    }
    if (p == &(vsCtl->c6tc)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_2;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6TC changed to %hd\n",
	    vsCtl->name, vsCtl->c6tc);
    }
    if (p == &(vsCtl->c6ts)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_3;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6TS changed to %f\n",
	    vsCtl->name, vsCtl->c6ts);
    }
    if (p == &(vsCtl->c6tg)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_4;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6TG changed to %f\n",
	    vsCtl->name, vsCtl->c6tg);
    }
    if (p == &(vsCtl->c6ta)) {
	vsCtl->zmfg |= ZMFG_TREND;
	vsCtl->zcfg |= ZCFG_6;
	vsCtl->zfg |= ZFG_5;
	if (recVacScanDebug >= 15)
	    printf("recVacScan.c: >%s< C6TA changed to %hd\n",
	    vsCtl->name, vsCtl->c6ta);
    }
    return (0);
}

static long get_graphic_double(DBADDR *paddr, struct dbr_grDouble *pgd)
{
    void *p;
    p = (void *) (paddr->pfield);
    recGblGetGraphicDouble(paddr, pgd);
    return (0);
}

static long get_control_double(DBADDR *paddr, struct dbr_ctrlDouble *pcd)
{
    void *p;
    p = (void *) (paddr->pfield);
    recGblGetControlDouble(paddr, pcd);
    return (0);
}

static long get_alarm_double(struct dbAddr *paddr, struct dbr_alDouble *pad)
{
    void *p;
    p = (void *) (paddr->pfield);
    pad->upper_alarm_limit = 0;
    pad->upper_warning_limit = 0;
    pad->lower_warning_limit = 0;
    pad->lower_alarm_limit = 0;
    return (0);
}

static void alarm(vacScanRecord *vsCtl)
{
    if (recVacScanDebug >= 25)
	printf("recVacScan.c: alarm()\n");
    if (vsCtl->udf == TRUE) {
	if (recVacScanDebug >= 20)
	    printf("recVacScan.c: udf set true... valid alarm\n");
	recGblSetSevr(vsCtl, UDF_ALARM, INVALID_ALARM);
	return;
    }
    return;
}

static void monitor(vacScanRecord *vsCtl)
{
    unsigned short monitor_mask;
    int alrm_chg_flg;
    alrm_chg_flg = 0;
    monitor_mask = 0;
    if (recVacScanDebug >= 25)
	printf("recVacScan.c: monitor()\n");
    monitor_mask = alrm_chg_flg = recGblResetAlarms(vsCtl);
    monitor_mask |= (DBE_VALUE | DBE_LOG);
    /*** check for value change ***/
    if ((vsCtl->ival != vsCtl->val) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->val, monitor_mask);
    if ((vsCtl->imod != vsCtl->rmod) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rmod, monitor_mask);
    if ((vsCtl->ibcf != vsCtl->rbcf) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rbcf, monitor_mask);
    if ((vsCtl->ibcs != vsCtl->rbcs) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rbcs, monitor_mask);
    if ((vsCtl->iafm != vsCtl->rafm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rafm, monitor_mask);
    if ((vsCtl->itri != vsCtl->rtri) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rtri, monitor_mask);
    if ((vsCtl->iprg != vsCtl->rprg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rprg, monitor_mask);
    if ((vsCtl->idet != vsCtl->rdet) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdet, monitor_mask);
    if ((vsCtl->iagn != vsCtl->ragn) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->ragn, monitor_mask);
    if ((vsCtl->itpr != vsCtl->rtpr) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rtpr, monitor_mask);
    if ((vsCtl->ignr != vsCtl->rgnr) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rgnr, monitor_mask);
    if ((vsCtl->issp != vsCtl->rssp) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rssp, monitor_mask);
    if ((vsCtl->ifs1 != vsCtl->rfs1) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rfs1, monitor_mask);
    if ((vsCtl->ifs2 != vsCtl->rfs2) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rfs2, monitor_mask);
    if ((vsCtl->ifsl != vsCtl->rfsl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rfsl, monitor_mask);
    if ((vsCtl->ifon != vsCtl->rfon) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rfon, monitor_mask);
    if ((vsCtl->idrs != vsCtl->rdrs) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdrs, monitor_mask);
    if ((vsCtl->idup != vsCtl->rdup) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdup, monitor_mask);
    if ((vsCtl->idbc != vsCtl->rdbc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdbc, monitor_mask);
    if ((vsCtl->idlc != vsCtl->rdlc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdlc, monitor_mask);
    if ((vsCtl->idmm != vsCtl->rdmm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdmm, monitor_mask);
    if ((vsCtl->idan != vsCtl->rdan) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdan, monitor_mask);
    if ((vsCtl->idvs != vsCtl->rdvs) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdvs, monitor_mask);
    if ((vsCtl->idut != vsCtl->rdut) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdut, monitor_mask);
    if ((vsCtl->iddt != vsCtl->rddt) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rddt, monitor_mask);
    if ((vsCtl->idal != vsCtl->rdal) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdal, monitor_mask);
    if ((vsCtl->iddg != vsCtl->rddg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rddg, monitor_mask);
    if ((vsCtl->idmc != vsCtl->rdmc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdmc, monitor_mask);
    if ((vsCtl->iddi != vsCtl->rddi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rddi, monitor_mask);
    if ((vsCtl->idtr != vsCtl->rdtr) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rdtr, monitor_mask);
    if ((vsCtl->ielm != vsCtl->nelm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->nelm, monitor_mask);
    if ((vsCtl->ifsc != vsCtl->wfsc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->wfsc, monitor_mask);
    if ((vsCtl->i1pm != vsCtl->r1pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pm, monitor_mask);
    if ((vsCtl->i1pv != vsCtl->r1pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pv, monitor_mask);
    if ((vsCtl->i1pd != vsCtl->r1pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pd, monitor_mask);
    if ((vsCtl->i1pa != vsCtl->r1pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pa, monitor_mask);
    if ((vsCtl->i1pg != vsCtl->r1pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pg, monitor_mask);
    if ((vsCtl->i1pi != vsCtl->r1pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pi, monitor_mask);
    if ((vsCtl->i1ph != vsCtl->r1ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1ph, monitor_mask);
    if ((vsCtl->i1pl != vsCtl->r1pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pl, monitor_mask);
    if ((vsCtl->i1pc != vsCtl->r1pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1pc, monitor_mask);
    if ((vsCtl->i1tm != vsCtl->r1tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1tm, monitor_mask);
    if ((vsCtl->i1tv != vsCtl->r1tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1tv, monitor_mask);
    if ((vsCtl->i1td != vsCtl->r1td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1td, monitor_mask);
    if ((vsCtl->i1ta != vsCtl->r1ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1ta, monitor_mask);
    if ((vsCtl->i1tg != vsCtl->r1tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1tg, monitor_mask);
    if ((vsCtl->i1ti != vsCtl->r1ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1ti, monitor_mask);
    if ((vsCtl->i1th != vsCtl->r1th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1th, monitor_mask);
    if ((vsCtl->i1tl != vsCtl->r1tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1tl, monitor_mask);
    if ((vsCtl->i1tc != vsCtl->r1tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r1tc, monitor_mask);
    if ((vsCtl->i2pm != vsCtl->r2pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pm, monitor_mask);
    if ((vsCtl->i2pv != vsCtl->r2pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pv, monitor_mask);
    if ((vsCtl->i2pd != vsCtl->r2pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pd, monitor_mask);
    if ((vsCtl->i2pa != vsCtl->r2pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pa, monitor_mask);
    if ((vsCtl->i2pg != vsCtl->r2pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pg, monitor_mask);
    if ((vsCtl->i2pi != vsCtl->r2pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pi, monitor_mask);
    if ((vsCtl->i2ph != vsCtl->r2ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2ph, monitor_mask);
    if ((vsCtl->i2pl != vsCtl->r2pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pl, monitor_mask);
    if ((vsCtl->i2pc != vsCtl->r2pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2pc, monitor_mask);
    if ((vsCtl->i2tm != vsCtl->r2tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2tm, monitor_mask);
    if ((vsCtl->i2tv != vsCtl->r2tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2tv, monitor_mask);
    if ((vsCtl->i2td != vsCtl->r2td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2td, monitor_mask);
    if ((vsCtl->i2ta != vsCtl->r2ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2ta, monitor_mask);
    if ((vsCtl->i2tg != vsCtl->r2tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2tg, monitor_mask);
    if ((vsCtl->i2ti != vsCtl->r2ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2ti, monitor_mask);
    if ((vsCtl->i2th != vsCtl->r2th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2th, monitor_mask);
    if ((vsCtl->i2tl != vsCtl->r2tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2tl, monitor_mask);
    if ((vsCtl->i2tc != vsCtl->r2tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r2tc, monitor_mask);
    if ((vsCtl->i3pm != vsCtl->r3pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pm, monitor_mask);
    if ((vsCtl->i3pv != vsCtl->r3pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pv, monitor_mask);
    if ((vsCtl->i3pd != vsCtl->r3pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pd, monitor_mask);
    if ((vsCtl->i3pa != vsCtl->r3pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pa, monitor_mask);
    if ((vsCtl->i3pg != vsCtl->r3pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pg, monitor_mask);
    if ((vsCtl->i3pi != vsCtl->r3pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pi, monitor_mask);
    if ((vsCtl->i3ph != vsCtl->r3ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3ph, monitor_mask);
    if ((vsCtl->i3pl != vsCtl->r3pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pl, monitor_mask);
    if ((vsCtl->i3pc != vsCtl->r3pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3pc, monitor_mask);
    if ((vsCtl->i3tm != vsCtl->r3tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3tm, monitor_mask);
    if ((vsCtl->i3tv != vsCtl->r3tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3tv, monitor_mask);
    if ((vsCtl->i3td != vsCtl->r3td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3td, monitor_mask);
    if ((vsCtl->i3ta != vsCtl->r3ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3ta, monitor_mask);
    if ((vsCtl->i3tg != vsCtl->r3tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3tg, monitor_mask);
    if ((vsCtl->i3ti != vsCtl->r3ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3ti, monitor_mask);
    if ((vsCtl->i3th != vsCtl->r3th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3th, monitor_mask);
    if ((vsCtl->i3tl != vsCtl->r3tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3tl, monitor_mask);
    if ((vsCtl->i3tc != vsCtl->r3tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r3tc, monitor_mask);
    if ((vsCtl->i4pm != vsCtl->r4pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pm, monitor_mask);
    if ((vsCtl->i4pv != vsCtl->r4pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pv, monitor_mask);
    if ((vsCtl->i4pd != vsCtl->r4pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pd, monitor_mask);
    if ((vsCtl->i4pa != vsCtl->r4pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pa, monitor_mask);
    if ((vsCtl->i4pg != vsCtl->r4pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pg, monitor_mask);
    if ((vsCtl->i4pi != vsCtl->r4pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pi, monitor_mask);
    if ((vsCtl->i4ph != vsCtl->r4ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4ph, monitor_mask);
    if ((vsCtl->i4pl != vsCtl->r4pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pl, monitor_mask);
    if ((vsCtl->i4pc != vsCtl->r4pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4pc, monitor_mask);
    if ((vsCtl->i4tm != vsCtl->r4tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4tm, monitor_mask);
    if ((vsCtl->i4tv != vsCtl->r4tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4tv, monitor_mask);
    if ((vsCtl->i4td != vsCtl->r4td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4td, monitor_mask);
    if ((vsCtl->i4ta != vsCtl->r4ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4ta, monitor_mask);
    if ((vsCtl->i4tg != vsCtl->r4tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4tg, monitor_mask);
    if ((vsCtl->i4ti != vsCtl->r4ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4ti, monitor_mask);
    if ((vsCtl->i4th != vsCtl->r4th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4th, monitor_mask);
    if ((vsCtl->i4tl != vsCtl->r4tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4tl, monitor_mask);
    if ((vsCtl->i4tc != vsCtl->r4tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r4tc, monitor_mask);
    if ((vsCtl->i5pm != vsCtl->r5pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pm, monitor_mask);
    if ((vsCtl->i5pv != vsCtl->r5pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pv, monitor_mask);
    if ((vsCtl->i5pd != vsCtl->r5pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pd, monitor_mask);
    if ((vsCtl->i5pa != vsCtl->r5pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pa, monitor_mask);
    if ((vsCtl->i5pg != vsCtl->r5pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pg, monitor_mask);
    if ((vsCtl->i5pi != vsCtl->r5pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pi, monitor_mask);
    if ((vsCtl->i5ph != vsCtl->r5ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5ph, monitor_mask);
    if ((vsCtl->i5pl != vsCtl->r5pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pl, monitor_mask);
    if ((vsCtl->i5pc != vsCtl->r5pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5pc, monitor_mask);
    if ((vsCtl->i5tm != vsCtl->r5tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5tm, monitor_mask);
    if ((vsCtl->i5tv != vsCtl->r5tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5tv, monitor_mask);
    if ((vsCtl->i5td != vsCtl->r5td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5td, monitor_mask);
    if ((vsCtl->i5ta != vsCtl->r5ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5ta, monitor_mask);
    if ((vsCtl->i5tg != vsCtl->r5tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5tg, monitor_mask);
    if ((vsCtl->i5ti != vsCtl->r5ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5ti, monitor_mask);
    if ((vsCtl->i5th != vsCtl->r5th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5th, monitor_mask);
    if ((vsCtl->i5tl != vsCtl->r5tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5tl, monitor_mask);
    if ((vsCtl->i5tc != vsCtl->r5tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r5tc, monitor_mask);
    if ((vsCtl->i6pm != vsCtl->r6pm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pm, monitor_mask);
    if ((vsCtl->i6pv != vsCtl->r6pv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pv, monitor_mask);
    if ((vsCtl->i6pd != vsCtl->r6pd) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pd, monitor_mask);
    if ((vsCtl->i6pa != vsCtl->r6pa) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pa, monitor_mask);
    if ((vsCtl->i6pg != vsCtl->r6pg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pg, monitor_mask);
    if ((vsCtl->i6pi != vsCtl->r6pi) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pi, monitor_mask);
    if ((vsCtl->i6ph != vsCtl->r6ph) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6ph, monitor_mask);
    if ((vsCtl->i6pl != vsCtl->r6pl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pl, monitor_mask);
    if ((vsCtl->i6pc != vsCtl->r6pc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6pc, monitor_mask);
    if ((vsCtl->i6tm != vsCtl->r6tm) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6tm, monitor_mask);
    if ((vsCtl->i6tv != vsCtl->r6tv) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6tv, monitor_mask);
    if ((vsCtl->i6td != vsCtl->r6td) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6td, monitor_mask);
    if ((vsCtl->i6ta != vsCtl->r6ta) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6ta, monitor_mask);
    if ((vsCtl->i6tg != vsCtl->r6tg) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6tg, monitor_mask);
    if ((vsCtl->i6ti != vsCtl->r6ti) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6ti, monitor_mask);
    if ((vsCtl->i6th != vsCtl->r6th) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6th, monitor_mask);
    if ((vsCtl->i6tl != vsCtl->r6tl) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6tl, monitor_mask);
    if ((vsCtl->i6tc != vsCtl->r6tc) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->r6tc, monitor_mask);
    if ((vsCtl->ic0v != vsCtl->rc0v)||(alrm_chg_flg))
	db_post_events(vsCtl,&vsCtl->rc0v,monitor_mask);
    if ((vsCtl->ic1v != vsCtl->rc1v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc1v, monitor_mask);
    if ((vsCtl->ic2v != vsCtl->rc2v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc2v, monitor_mask);
    if ((vsCtl->ic3v != vsCtl->rc3v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc3v, monitor_mask);
    if ((vsCtl->ic4v != vsCtl->rc4v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc4v, monitor_mask);
    if ((vsCtl->ic5v != vsCtl->rc5v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc5v, monitor_mask);
    if ((vsCtl->ic6v != vsCtl->rc6v) || (alrm_chg_flg))
	db_post_events(vsCtl, &vsCtl->rc6v, monitor_mask);
    return;
}
