// vacScanServer.cc

// Author: Greg Nawrocki
// Date: 1/97
// Mods for MPF instead of Hideos Marty Kraimer
// Date: 19AUG98

extern "C" {
#include <vxWorks.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <taskLib.h>
#include <semLib.h>
}
 
#include "Message.h"
#include "Int32Message.h"
#include "Char8ArrayMessage.h"
#include "SerialPort.h"
#include "vacScanDefs.h"

int vacScanDebug = 1;

#define        MODE_PEAK   1
#define        MODE_BAR    2
#define        MODE_LEAK   5
#define        MODE_ANALOG 7
#define        MODE_TREND  19

static const int numChannel = 6;
static const int maxMass = 200;
static const int peakHeightSize = 10;
static const int readBufferSize = 1000;
static const int responseBufferSize = 1000;
static const int timeoutSeconds = 5;
 
enum readStatus
{
    readSuccess=0, readTimeout, readNak, readOverrun,
    readBadTerm, readFailure
};

static const char * const readStatusString[] = 
{
    "readSuccess","readTimeout","readNak","readOverrun",
    "readBadTerm","readFailure"
};

enum readState { readIdle, readStart, readMessage, readGotDelim};

struct VacscanStatus {
    int barChartFirstMass;
    int barChartSpan;
    int analogFirstMass;
    int trendInterval;
    int status[2];
};

struct ChannelStatus {
    int massNumber;
    int alarmLevel;
    int status[2];
};

class VacScan
{
public:
    VacScan(const char *name,const char *portName, int queueSize);
    static void vacScanServer(VacScan *);
    bool getStartOk() const {return(startOk);}
    static byteHandlerRC byteHandler(void*,unsigned char);
    void talk(Char8ArrayMessage *pmessage);
private:
    bool xact(const char* cmdBuf,int nCrLf);
    bool getVacscanStatus();
    bool getChannelStatus();
    bool getTotalPressure();
    void getNumberPeaks();
    bool getPeakHeight();
    MessageServer *pMessageServer;
    SerialPort* pSerialPort;
    bool startOk;
    int currentMode;
    VacscanStatus vacscanStatus;
    ChannelStatus channelStatus[numChannel];
    int numberPeaks;
    char peakHeight[maxMass][peakHeightSize];
    readStatus status;
    readState state;
    int nextRead;
    int delimRemaining;
    char readBuf[readBufferSize];
    int nextResponse;
    char responseBuf[responseBufferSize];
};

static char taskname[] = "vacScan";
int initVacScanServer(char *serverName,char *portName,int queueSize)
{
    VacScan *pVacScan = new VacScan(serverName,portName,queueSize);
    if(!pVacScan->getStartOk()) return(0);
    int taskId = taskSpawn(taskname,100,VX_FP_TASK,2000,
	(FUNCPTR)VacScan::vacScanServer,(int)pVacScan,0,0,0,0,0,0,0,0,0);
    if(taskId==ERROR) 
	printf("vacScanServer taskSpawn Failure\n");
    return(0);
}
	

VacScan:: VacScan(const char *name,const char *portName, int queueSize)
: startOk(false), currentMode(0), numberPeaks(0), status(readFailure),
  state(readIdle), nextRead(0), delimRemaining(0), nextResponse(0)
{
    ::memset(&vacscanStatus,0,sizeof(vacscanStatus));
    for(int i=0; i<numChannel; i++)
	::memset(&channelStatus[i],0,sizeof(channelStatus));
    for(int i=0; i<maxMass; i++) strcpy(peakHeight[i],"");
    pSerialPort = SerialPort::bind(portName,this,byteHandler);
    if(pSerialPort==0) {
	printf("%s: could not bind to SerialPort %s\n",name,portName);
        return;
    }
    pMessageServer = new MessageServer(name,queueSize);
    startOk = true;
}

void VacScan::vacScanServer(VacScan *pVacScan)
{
    while(true) {
        pVacScan->pMessageServer->waitForMessage();
	Message *message;
        while((message = pVacScan->pMessageServer->receive())) {
            if(message->getType()!=messageTypeChar8Array) {
                printf("vacScanServer got illegal message type %d\n",
                    message->getType());
		continue;
            }
            Char8ArrayMessage  *preceive = (Char8ArrayMessage *)message;
	    pVacScan->talk(preceive);
	    Message *psend;
	    if(pVacScan->status!=readSuccess) {
		Int32Message *pm = (Int32Message *)pVacScan->pMessageServer->
			allocReplyMessage(preceive,messageTypeInt32);
		pm->status = -1;
		psend = pm;
	    } else {
                Char8ArrayMessage *pm =
			(Char8ArrayMessage *)pVacScan->pMessageServer->
                        allocReplyMessage(preceive,messageTypeChar8Array);
		pm->allocValue(pVacScan->nextResponse);
		::memcpy(pm->value,pVacScan->responseBuf,pVacScan->nextResponse);
		pm->setSize(pVacScan->nextResponse);
		psend = pm;
	    }
            delete preceive;
            pVacScan->pMessageServer->reply(psend);
	}
    }
    return;
}

byteHandlerRC VacScan::byteHandler(void* v, unsigned char indata)
{
    register char data = (char)indata;
    VacScan* pVacScan = (VacScan*)v;
    switch(pVacScan->state) {
    case readIdle:
        break;
    case readStart:
	if(data=='A') {
	    pVacScan->state = readMessage;
	} else if(data=='N') {
	    pVacScan->state = readIdle;
	    pVacScan->status = readNak;
	    return(byteHandlerEndRead);
	}
        break;
    case readMessage:
	if(data=='\r') {
	    pVacScan->state=readGotDelim;
	} else {
	    if(pVacScan->nextRead >= (readBufferSize-1)) {
		pVacScan->state=readIdle;
	        pVacScan->status = readOverrun;
	        return(byteHandlerEndRead);
	    } else {
                pVacScan->readBuf[pVacScan->nextRead++]=data;
	    }
	}
        break;
    case readGotDelim:
	if(data != '\n') {
	    pVacScan->status = readBadTerm;
	    return(byteHandlerEndRead);
	}
	if(--pVacScan->delimRemaining > 0 ) {
	    pVacScan->readBuf[pVacScan->nextRead++]=',';
	    pVacScan->state = readMessage;
	} else {
	    pVacScan->state=readIdle;
	    pVacScan->status = readSuccess;
	    return(byteHandlerEndRead);
	}
	break;
    default:
        break;
    }
    return(byteHandlerOK);
}

bool VacScan::xact(const char* cmdBuf,int nCrLf)
{
    serialPortStatus ioStatus;

    state = readStart; nextRead=0; delimRemaining = nCrLf;
    ioStatus = pSerialPort->writeRead(
	(unsigned char *)cmdBuf,::strlen(cmdBuf),timeoutSeconds);
    if(ioStatus==serialPortTimeout) status = readTimeout;
    readBuf[nextRead] = 0;
    if((vacScanDebug>1) || (status!=readSuccess && vacScanDebug)) { 
	printf("%s %s cmd %s reply %s\n",
	    pMessageServer->getName(),readStatusString[status],
	    cmdBuf, readBuf);
    }
    return((status==readSuccess) ? true : false);
}

bool VacScan::getVacscanStatus()
{
    if(!xact("(S)",1)) return(false);
    int nfields = sscanf(readBuf,"%i %i %i %i %x %x",
	&vacscanStatus.barChartFirstMass,&vacscanStatus.barChartSpan,
	&vacscanStatus.analogFirstMass,&vacscanStatus.trendInterval,
	&vacscanStatus.status[0],&vacscanStatus.status[1]);
    if(nfields!=6) {
	printf("%s getVacscanStatus failed reply %s\n",
	    pMessageServer->getName(),readBuf);
	status = readFailure;
	return(false);
    }
    if(vacScanDebug>1) {
        printf("getVacscanStatus readBuf %s\n",readBuf);
        printf("barChartFirstMass %d\n",vacscanStatus.barChartFirstMass);
        printf("barChartSpan %d\n",vacscanStatus.barChartSpan);
        printf("analogFirstMass %d\n",vacscanStatus.analogFirstMass);
        printf("trendInterval %d\n",vacscanStatus.trendInterval);
        printf("status %4.4x %4.4x\n",
	    vacscanStatus.status[0],vacscanStatus.status[1]);
    }
    return(true);
}

bool VacScan::getChannelStatus()
{
    for(int channel=0; channel<6; channel++) {
	char message[10];
	sprintf(message,"(C%d)",channel+1);
        if(!xact(message,1)) return(false);
        int nfields = sscanf(readBuf,"%i %i %x %x",
	    &channelStatus[channel].massNumber,
	    &channelStatus[channel].alarmLevel,
	    &channelStatus[channel].status[0],
	    &channelStatus[channel].status[1]);
        if(nfields!=4) {
	    printf("%s getChannelStatus failed reply %s\n",
	        pMessageServer->getName(),readBuf);
	    status = readFailure;
	    return(false);
        }
	if(vacScanDebug>1) {
            printf("getChannelStatus %d readBuf %s\n",channel,readBuf);
            printf(" massNumber %d\n",channelStatus[channel].massNumber);
            printf(" alarmLevel %d\n",channelStatus[channel].alarmLevel);
            printf("status %4.4x %4.4x\n",
	        channelStatus[channel].status[0],
		channelStatus[channel].status[1]);
	}
    }
    return(true);
}

bool VacScan::getTotalPressure()
{
    if(currentMode!=MODE_PEAK
    && currentMode!=MODE_BAR
    && currentMode!=MODE_LEAK
    && currentMode!=MODE_ANALOG
    && currentMode!=MODE_TREND ) return(false);
    if(!xact("(P0)",1)) return(false);
    if(vacScanDebug>1) printf("getTotalPressure %s\n",readBuf);
    return(true);
}

void VacScan::getNumberPeaks()
{
    switch(currentMode) {
    case MODE_BAR:
	numberPeaks = vacscanStatus.barChartSpan;
	break;
    case MODE_PEAK:
    case MODE_TREND:
        numberPeaks = 6;
        break;
    case MODE_LEAK:
	numberPeaks = 1;
	break;
    default:
	numberPeaks = 0;
        break;
    }
}

bool VacScan::getPeakHeight()
{
    if(currentMode==MODE_BAR) {
        if(!xact("(P)",numberPeaks)) return(false);
        char *pnext = readBuf;
	char *pE = ::strchr(pnext,'E');
	if(!pE) {
	    printf("%s getPeakHeight cant find E\n",pMessageServer->getName());
	    status = readFailure; return(false);
	}
        char expPart[10];
        ::strcpy(expPart,pE );
	strcpy(pE,",");
        char *pcomma = pnext;
        for(int ind=0; ind<numberPeaks; ind++) {
	    pcomma = ::strchr(pnext,',');
	    if(!pcomma) {
	        printf("%s getPeakHeight cant find ,\n",pMessageServer->getName());
		status = readFailure; return(false);
	    }
	    *pcomma = 0; //change to end of string
	    pcomma += 1;
	    ::strcpy(peakHeight[ind],pnext);
	    pnext = pcomma;
	}
        for(int ind=0; ind<numberPeaks; ind++)
	    ::strcat(peakHeight[ind],expPart);
    } else {
	if(numberPeaks<=0) {
	    status = readFailure; return(false);
	}
        if(!xact("(P)",numberPeaks)) return(false);
        char *pnext = readBuf;
        char *pcomma = pnext;
        for(int ind=0; ind<numberPeaks; ind++) {
	    if(ind < numberPeaks -1) {
	        pcomma = ::strchr(pnext,',');
	        if(!pcomma) {
	            printf("%s getPeakHeight cant find comma\n",
			pMessageServer->getName());
		    status = readFailure; return(false);
	        }
	        *pcomma = 0; //change to end of string
	        pcomma += 1;
	    }
	    ::strcpy(peakHeight[ind],pnext);
	    pnext = pcomma;
	}
    }
    if(vacScanDebug>1) {
	printf("getPeakHeight");
	for(int ind=0; ind<numberPeaks; ind++) {
	    if(ind%10 == 0) printf("\n"); else printf(" ");
	    printf("%s",peakHeight[ind]);
	}
	printf("\n");
    }
    return(true);
}

void VacScan::talk(Char8ArrayMessage* pinmessage)
{
    if(!xact("(A1)",1)) return;
    cmdType type = (cmdType)pinmessage->cmd;
    if(type == cmdPutGetCtl) {
	char message[20];
	::strcpy(message,"(");
	::strcat(message,pinmessage->value);
	::strcat(message,")");
	if(!xact(message,1)) return;
    }
    if(!xact("(m)",1)) return;
    currentMode = -1;
    sscanf(readBuf,"%d",&currentMode);
    if(currentMode<0) {
        printf("%s get mode failed. reply %s\n",
            pMessageServer->getName(),readBuf);
        status = readFailure;
        return;
    }
    if(!getVacscanStatus()) return;
    getNumberPeaks();
    if(type==cmdGetPeak) {
	if(!getPeakHeight()) return;
	responseBuf[0] = 0;
	for(int i=0; i<numberPeaks; i++) {
	    ::strcat(responseBuf,peakHeight[i]);
	    if( i!= numberPeaks-1) ::strcat(responseBuf,",");
	}
	nextResponse = ::strlen(responseBuf);
	status = readSuccess;
	return;
    }
    if(type==cmdGetX) {
	responseBuf[0] = 0;
	for(int i=0; i<numberPeaks; i++) {
	    char value[10];
	    sprintf(value,"%d",i+vacscanStatus.barChartFirstMass);
	    ::strcat(responseBuf,value);
	    if( i!= numberPeaks-1) ::strcat(responseBuf,",");
	}
	nextResponse = ::strlen(responseBuf);
	status = readSuccess;
	return;
    }

    /* get response data */
    ::memset(responseBuf,0,200);
    responseBuf[0] = (char)currentMode;
    responseBuf[1] = (char)vacscanStatus.barChartFirstMass;
    responseBuf[2] = (char)vacscanStatus.barChartSpan;
    responseBuf[3] = (char)vacscanStatus.analogFirstMass;
    responseBuf[4] = (char)vacscanStatus.trendInterval;
    responseBuf[5] = (char)((vacscanStatus.status[0]>>8) & 0xFF);
    responseBuf[6] = (char)(vacscanStatus.status[0] & 0xFF);
    responseBuf[7] = (char)((vacscanStatus.status[1]>>8) & 0xFF);
    responseBuf[8] = (char)(vacscanStatus.status[1] & 0xFF);
    if(currentMode==MODE_PEAK || currentMode==MODE_TREND) {
        if(getChannelStatus())  {
            for(int channel=0; channel<6; channel++) {
	        int offset = 10*(channel+1);
	        responseBuf[offset] = (char)channelStatus[channel].massNumber;
	        responseBuf[offset+1] =
		    (char)channelStatus[channel].alarmLevel;
	        responseBuf[offset+2] =
		    (char)(channelStatus[channel].status[0]&0xFF);
	        responseBuf[offset+3] =
		    (char)(channelStatus[channel].status[1]&0xFF);
	    }
	} else {
	    printf("%s getChannelStatus failed\n",pMessageServer->getName());
	}
	if(getPeakHeight()) {
	    responseBuf[70] = (char)numberPeaks;
	    for(int i=0; i<numberPeaks; i++)
		::strcpy(&responseBuf[100+i*10],peakHeight[i]);
	} else {
	  printf("%s getPeakHeight failed\n",pMessageServer->getName());
	}
    }
    if(currentMode==MODE_LEAK) {
        // get probe gas
        if(xact("(s14)",1)) {
	    responseBuf[9] = (char)atoi(readBuf);
	    if(vacScanDebug>1) printf("probe gas %2.2x\n",responseBuf[9]);
        } else {
	    printf("%s get probe gas failed\n",pMessageServer->getName());
	}
	if(getPeakHeight()) {
	    responseBuf[70] = (char)numberPeaks;
	    for(int i=0; i<numberPeaks; i++)
		::strcpy(&responseBuf[100+i*10],peakHeight[i]);
	} else {
	  printf("%s getPeakHeight failed\n",pMessageServer->getName());
	}
    }
    if(getTotalPressure()) ::strcpy(&responseBuf[90],readBuf);
    nextResponse = 160;
    status = readSuccess;
}
